from truegaze.plugins.adobe_mobile_sdk import  AdobeMobileSdkPlugin

# List of active plugins - when developing a new plugin, it should be added here.
# BasePlugin should never be added to this list.
ACTIVE_PLUGINS = [
    AdobeMobileSdkPlugin
]